@@ -105,6 +105,33 @@ pub struct CompilerConfig {
     pub plugins: Vec<Box<dyn Plugin>>,
 }
 
+pub fn load_schema_or_bundle(bundle: &mut Map<ModulePath, Schema>, i: &PathBuf) -> io::Result<()> {
+    let mut f = File::open(&i)?;
+    let mut src = IOBinarySource::new(&mut f);
+    let mut reader = src.packed_iovalues();
+    let blob = reader.demand_next(false)?;
+    let language = Language::default();
+
+    if let Ok(s) = language.parse(&blob) {
+        let prefix = i.file_stem().ok_or_else(
+            || io::Error::new(io::ErrorKind::InvalidData,
+                              format!("Bad schema file stem: {:?}", i)))?
+            .to_str().ok_or_else(
+                || io::Error::new(io::ErrorKind::InvalidData,
+                                  format!("Invalid UTF-8 in schema file name: {:?}", i)))?;
+        bundle.insert(vec![prefix.to_owned()], s);
+    } else if let Ok(Bundle { modules }) = language.parse(&blob) {
+        for (ModulePath(k), v) in modules.0 {
+            bundle.insert(k, v);
+        }
+    } else {
+        return Err(io::Error::new(io::ErrorKind::InvalidData,
+                                  format!("Invalid schema binary blob {:?}", i)));
+    }
+
+    Ok(())
+}
+
 impl CompilerConfig {
     pub fn new(
         output_dir: PathBuf,
@@ -134,32 +161,7 @@ impl CompilerConfig {
 
     pub fn load_schemas_and_bundles(&mut self, inputs: &Vec<PathBuf>) -> io::Result<()> {
         for i in inputs {
-            let mut f = File::open(&i)?;
-            let mut src = IOBinarySource::new(&mut f);
-            let mut reader = src.packed_iovalues();
-            let blob = reader.demand_next(false)?;
-            let language = Language::default();
-
-            if let Ok(s) = language.parse(&blob) {
-                let prefix = i.file_stem().ok_or_else(
-                    || io::Error::new(io::ErrorKind::InvalidData,
-                                      format!("Bad schema file stem: {:?}", i)))?
-                    .to_str().ok_or_else(
-                        || io::Error::new(io::ErrorKind::InvalidData,
-                                          format!("Invalid UTF-8 in schema file name: {:?}", i)))?;
-                self.bundle.insert(vec![prefix.to_owned()], s);
-                continue;
-            }
-
-            if let Ok(Bundle { modules }) = language.parse(&blob) {
-                for (ModulePath(k), v) in modules.0 {
-                    self.bundle.insert(k, v);
-                }
-                continue;
-            }
-
-            return Err(io::Error::new(io::ErrorKind::InvalidData,
-                                      format!("Invalid schema binary blob {:?}", i)));
+            load_schema_or_bundle(&mut self.bundle, i)?;
         }
         Ok(())
     }
