@@ -4,29 +4,69 @@ pub use preserves;
 pub use preserves::value::Reader;
 pub use preserves::value::boundary as B;
 
+pub mod interpret;
+
 use preserves::value::ArcValue;
 use preserves::value::Domain;
-use preserves::value::Embeddable;
 use preserves::value::IOValue;
 use preserves::value::NestedValue;
 use preserves::value::NoEmbeddedDomainCodec;
 use preserves::value::Value;
 
 use std::convert::From;
+use std::convert::Into;
 use std::convert::TryFrom;
 use std::io;
 use std::sync::Arc;
 
 use thiserror::Error;
 
-pub trait Deserialize<'de, D: Embeddable, N: NestedValue<D>, R: Reader<'de, D, N>>
+pub trait NestedValueCodec {} // marker trait
+impl NestedValueCodec for () {}
+
+pub trait Parse<L, Value: NestedValue>: Sized {
+    fn parse(language: L, value: &Value) -> Result<Self, ParseError>;
+}
+
+impl<'a, T: NestedValueCodec, Value: NestedValue> Parse<&'a T, Value> for Value {
+    fn parse(_language: &'a T, value: &Value) -> Result<Self, ParseError> {
+        Ok(value.clone())
+    }
+}
+
+pub trait Unparse<L, Value: NestedValue> {
+    fn unparse(&self, language: L) -> Value;
+}
+
+impl<'a, T: NestedValueCodec, Value: NestedValue> Unparse<&'a T, Value> for Value {
+    fn unparse(&self, _language: &'a T) -> Value {
+        self.clone()
+    }
+}
+
+pub trait Codec<N: NestedValue> {
+    fn parse<'a, T: Parse<&'a Self, N>>(&'a self, value: &N) -> Result<T, ParseError>;
+    fn unparse<'a, T: Unparse<&'a Self, N>>(&'a self, value: &T) -> N;
+}
+
+impl<L, N: NestedValue> Codec<N> for L {
+    fn parse<'a, T: Parse<&'a L, N>>(&'a self, value: &N) -> Result<T, ParseError> {
+        T::parse(self, value)
+    }
+
+    fn unparse<'a, T: Unparse<&'a L, N>>(&'a self, value: &T) -> N {
+        value.unparse(self)
+    }
+}
+
+pub trait Deserialize<'de, N: NestedValue, R: Reader<'de, N>>
 where
     Self: Sized
 {
     fn deserialize(r: &mut R) -> Result<Self, ParseError>;
 }
 
-pub fn decode_lit<D: Embeddable, N: NestedValue<D>>(bs: &[u8]) -> io::Result<N> {
+pub fn decode_lit<N: NestedValue>(bs: &[u8]) -> io::Result<N> {
     preserves::value::packed::from_bytes(bs, NoEmbeddedDomainCodec)
 }
 
@@ -45,7 +85,7 @@ pub fn encode_embedded<D: Domain>(
 where
     for<'a> IOValue: From<&'a D>
 {
-    v.copy_via::<_, _, _, std::convert::Infallible>(
+    v.copy_via::<_, _, std::convert::Infallible>(
         &mut |d| Ok(Value::Embedded(IOValue::from(d)))).unwrap()
 }
 
